@@ -1,7 +1,8 @@
-// $Id$
-
-// Copyright by Johannes Sixt
-// This file is under GPL, the GNU General Public Licence
+/*
+ * Copyright Johannes Sixt
+ * This file is licensed under the GNU General Public License Version 2.
+ * See the file COPYING in the toplevel directory of the source directory.
+ */
 
 #include "debugger.h"
 #include "dbgdriver.h"
@@ -1725,24 +1726,23 @@ void KDebugger::evalInitialStructExpression(VarTree* var, ExprWnd* wnd, bool imm
 void KDebugger::evalStructExpression(VarTree* var, ExprWnd* wnd, bool immediate)
 {
     QString base = var->computeExpr();
-    QString exprFmt;
+    QString expr;
     if (var->m_exprIndexUseGuard) {
-	exprFmt = var->m_type->m_guardStrings[var->m_exprIndex];
-	if (exprFmt.isEmpty()) {
+	expr = var->m_type->m_guardStrings[var->m_exprIndex];
+	if (expr.isEmpty()) {
 	    // no guard, omit it and go to expression
 	    var->m_exprIndexUseGuard = false;
 	}
     }
     if (!var->m_exprIndexUseGuard) {
-	exprFmt = var->m_type->m_exprStrings[var->m_exprIndex];
+	expr = var->m_type->m_exprStrings[var->m_exprIndex];
     }
 
-    QString expr;
-    expr.sprintf(exprFmt, base.data());
+    expr.replace("%s", base);
 
     DbgCommand dbgCmd = DCprintStruct;
     // check if this is a QString::Data
-    if (strncmp(expr, "/QString::Data ", 15) == 0)
+    if (expr.left(15) == "/QString::Data ")
     {
 	if (m_typeTable->parseQt2QStrings())
 	{
@@ -1758,8 +1758,6 @@ void KDebugger::evalStructExpression(VarTree* var, ExprWnd* wnd, bool immediate)
 	    // TODO: remove this "print"; queue the next printStruct instead
 	    expr = "*0";
 	}
-    } else {
-	expr = expr;
     }
     TRACE("evalStruct: " + expr + (var->m_exprIndexUseGuard ? " // guard" : " // real"));
     CmdQueueItem* cmd = m_d->queueCmd(dbgCmd, expr,
@@ -1826,7 +1824,7 @@ void KDebugger::slotDeleteWatch()
 {
     // delete only allowed while debugger is idle; or else we might delete
     // the very expression the debugger is currently working on...
-    if (!m_d->isIdle())
+    if (m_d == 0 || !m_d->isIdle())
 	return;
 
     VarTree* item = m_watchVariables.currentItem();
@@ -2096,10 +2094,12 @@ void KDebugger::slotValuePopup(const QString& expr)
 
 void KDebugger::slotDisassemble(const QString& fileName, int lineNo)
 {
-    CmdQueueItem* cmd = m_d->queueCmd(DCinfoline, fileName, lineNo,
-				      DebuggerDriver::QMoverrideMoreEqual);
-    cmd->m_fileName = fileName;
-    cmd->m_lineNo = lineNo;
+    if (m_haveExecutable) {
+	CmdQueueItem* cmd = m_d->queueCmd(DCinfoline, fileName, lineNo,
+					  DebuggerDriver::QMoverrideMoreEqual);
+	cmd->m_fileName = fileName;
+	cmd->m_lineNo = lineNo;
+    }
 }
 
 void KDebugger::handleInfoLine(CmdQueueItem* cmd, const char* output)
